@@ -74,7 +74,7 @@ void ElisaConfigurationDialog::save()
     Elisa::ElisaConfiguration::setRootPath(mRootPath);
     Elisa::ElisaConfiguration::setShowProgressOnTaskBar(mShowProgressInTaskBar);
     Elisa::ElisaConfiguration::setShowSystemTrayIcon(mShowSystemTrayIcon);
-    Elisa::ElisaConfiguration::setForceUsageOfFastFileSearch(mForceUsageOfFastFileSearch);
+    Elisa::ElisaConfiguration::setForceUsageOfFastFileSearch(!mForceUsageOfSlowFileSystemIndexing);
     Elisa::ElisaConfiguration::setPlayAtStartup(mPlayAtStartup);
     Elisa::ElisaConfiguration::setColorScheme(mColorScheme);
 
@@ -110,6 +110,16 @@ void ElisaConfigurationDialog::save()
     Q_EMIT isDirtyChanged();
 }
 
+void ElisaConfigurationDialog::cancel()
+{
+    if (mIsDirty) {
+        configChanged();
+
+        mIsDirty = false;
+        Q_EMIT isDirtyChanged();
+    }
+}
+
 void ElisaConfigurationDialog::setShowProgressInTaskBar(bool showProgressInTaskBar)
 {
     if (mShowProgressInTaskBar == showProgressInTaskBar) {
@@ -134,14 +144,14 @@ void ElisaConfigurationDialog::setShowSystemTrayIcon(bool showSystemTrayIcon)
     setDirty();
 }
 
-void ElisaConfigurationDialog::setForceUsageOfFastFileSearch(bool forceUsageOfFastFileSearch)
+void ElisaConfigurationDialog::setForceUsageOfSlowFileSystemIndexing(bool forceUsageOfSlowFileSystemIndexing)
 {
-    if (mForceUsageOfFastFileSearch == forceUsageOfFastFileSearch) {
+    if (mForceUsageOfSlowFileSystemIndexing == forceUsageOfSlowFileSystemIndexing) {
         return;
     }
 
-    mForceUsageOfFastFileSearch = forceUsageOfFastFileSearch;
-    Q_EMIT forceUsageOfFastFileSearchChanged();
+    mForceUsageOfSlowFileSystemIndexing = forceUsageOfSlowFileSystemIndexing;
+    Q_EMIT forceUsageOfSlowFileSystemIndexingChanged();
 
     setDirty();
 }
@@ -192,32 +202,51 @@ void ElisaConfigurationDialog::setColorScheme(const QString &scheme)
     setDirty();
 }
 
+void ElisaConfigurationDialog::removeMusicLocation(QString location)
+{
+    mRootPath.removeAll(location);
+    emit rootPathChanged(mRootPath);
+}
+
 void ElisaConfigurationDialog::configChanged()
 {
-    setRootPath(Elisa::ElisaConfiguration::rootPath());
-    setShowProgressInTaskBar(Elisa::ElisaConfiguration::showProgressOnTaskBar());
-    setShowSystemTrayIcon(Elisa::ElisaConfiguration::showSystemTrayIcon());
-    setPlayAtStartup(Elisa::ElisaConfiguration::playAtStartup());
-    setColorScheme(Elisa::ElisaConfiguration::colorScheme());
-    setForceUsageOfFastFileSearch(Elisa::ElisaConfiguration::forceUsageOfFastFileSearch());
+    mRootPath = Elisa::ElisaConfiguration::rootPath();
+    Q_EMIT rootPathChanged(mRootPath);
+
+    mShowProgressInTaskBar = Elisa::ElisaConfiguration::showProgressOnTaskBar();
+    Q_EMIT showProgressInTaskBarChanged();
+
+    mShowSystemTrayIcon = Elisa::ElisaConfiguration::showSystemTrayIcon();
+    Q_EMIT showSystemTrayIconChanged();
+
+    mPlayAtStartup = Elisa::ElisaConfiguration::playAtStartup();
+    Q_EMIT playAtStartupChanged();
+
+    mColorScheme = Elisa::ElisaConfiguration::colorScheme();
+    Q_EMIT colorSchemeChanged();
+
+    mForceUsageOfSlowFileSystemIndexing = !Elisa::ElisaConfiguration::forceUsageOfFastFileSearch();
+    Q_EMIT forceUsageOfSlowFileSystemIndexingChanged();
 
     switch (Elisa::ElisaConfiguration::embeddedView())
     {
     case Elisa::ElisaConfiguration::EnumEmbeddedView::NoView:
-        setEmbeddedView(ElisaUtils::Unknown);
+        mEmbeddedView = ElisaUtils::Unknown;
         break;
     case Elisa::ElisaConfiguration::EnumEmbeddedView::AllAlbums:
-        setEmbeddedView(ElisaUtils::Album);
+        mEmbeddedView = ElisaUtils::Album;
         break;
     case Elisa::ElisaConfiguration::EnumEmbeddedView::AllArtists:
-        setEmbeddedView(ElisaUtils::Artist);
+        mEmbeddedView = ElisaUtils::Artist;
         break;
     case Elisa::ElisaConfiguration::EnumEmbeddedView::AllGenres:
-        setEmbeddedView(ElisaUtils::Genre);
+        mEmbeddedView = ElisaUtils::Genre;
         break;
     }
+    Q_EMIT embeddedViewChanged();
 
-    setInitialViewIndex(Elisa::ElisaConfiguration::initialView());
+    mInitialViewIndex = Elisa::ElisaConfiguration::initialView();
+    Q_EMIT initialViewIndexChanged();
 }
 
 void ElisaConfigurationDialog::setDirty()
