@@ -1,6 +1,7 @@
 /*
    SPDX-FileCopyrightText: 2015 (c) Matthieu Gallien <matthieu_gallien@yahoo.fr>
    SPDX-FileCopyrightText: 2019 (c) Alexander Stippich <a.stippich@gmx.net>
+   SPDX-FileCopyrightText: 2020 (c) Carson Black <uhhadd@gmail.com>
 
    SPDX-License-Identifier: LGPL-3.0-or-later
  */
@@ -25,6 +26,16 @@ class ELISALIB_EXPORT MediaPlayListProxyModel : public QAbstractProxyModel
 
     Q_OBJECT
 
+public:
+    enum Repeat {
+        None,
+        One,
+        Playlist
+    };
+    Q_ENUM(Repeat)
+
+private:
+
     Q_PROPERTY(QVariantMap persistentState
                READ persistentState
                WRITE setPersistentState
@@ -42,10 +53,10 @@ class ELISALIB_EXPORT MediaPlayListProxyModel : public QAbstractProxyModel
                READ nextTrack
                NOTIFY nextTrackChanged)
 
-    Q_PROPERTY(bool repeatPlay
-               READ repeatPlay
-               WRITE setRepeatPlay
-               NOTIFY repeatPlayChanged)
+    Q_PROPERTY(Repeat repeatMode
+               READ repeatMode
+               WRITE setRepeatMode
+               NOTIFY repeatModeChanged)
 
     Q_PROPERTY(bool shufflePlayList
                READ shufflePlayList
@@ -70,47 +81,50 @@ public:
 
     ~MediaPlayListProxyModel() override;
 
-    QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const override;
+    [[nodiscard]] QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const override;
 
-    QModelIndex mapFromSource(const QModelIndex &sourceIndex) const override;
+    [[nodiscard]] QModelIndex mapFromSource(const QModelIndex &sourceIndex) const override;
 
-    QItemSelection mapSelectionFromSource(const QItemSelection &sourceSelection) const override;
+    [[nodiscard]] QItemSelection mapSelectionFromSource(const QItemSelection &sourceSelection) const override;
 
-    QItemSelection mapSelectionToSource(const QItemSelection &proxySelection) const override;
+    [[nodiscard]] QItemSelection mapSelectionToSource(const QItemSelection &proxySelection) const override;
 
-    QModelIndex mapToSource(const QModelIndex &proxyIndex) const override;
+    [[nodiscard]] QModelIndex mapToSource(const QModelIndex &proxyIndex) const override;
 
-    int mapRowFromSource(const int sourceRow) const;
+    [[nodiscard]] int mapRowFromSource(const int sourceRow) const;
 
-    int mapRowToSource(const int proxyRow) const;
+    [[nodiscard]] int mapRowToSource(const int proxyRow) const;
 
-    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+    [[nodiscard]] int rowCount(const QModelIndex &parent = QModelIndex()) const override;
 
-    int columnCount(const QModelIndex &parent) const override;
+    [[nodiscard]] int columnCount(const QModelIndex &parent) const override;
 
-    QModelIndex parent(const QModelIndex &child) const override;
+    [[nodiscard]] QModelIndex parent(const QModelIndex &child) const override;
 
-    bool hasChildren(const QModelIndex &parent) const override;
+    [[nodiscard]] bool hasChildren(const QModelIndex &parent) const override;
 
     void setPlayListModel(MediaPlayList* playListModel);
 
-    QPersistentModelIndex previousTrack() const;
+    [[nodiscard]] QPersistentModelIndex previousTrack() const;
 
-    QPersistentModelIndex currentTrack() const;
+    [[nodiscard]] QPersistentModelIndex currentTrack() const;
 
-    QPersistentModelIndex nextTrack() const;
+    [[nodiscard]] QPersistentModelIndex nextTrack() const;
 
-    bool repeatPlay() const;
+    [[nodiscard]] Repeat repeatMode() const;
 
-    bool shufflePlayList() const;
+    [[nodiscard]] bool shufflePlayList() const;
 
-    int remainingTracks() const;
+    [[nodiscard]] int remainingTracks() const;
 
-    int currentTrackRow() const;
+    [[nodiscard]] int currentTrackRow() const;
 
-    int tracksCount() const;
+    [[nodiscard]] int tracksCount() const;
+
+    [[nodiscard]] QVariantMap persistentState() const;
+
+    int mSeekToBeginningDelay = 2000;
 
-    QVariantMap persistentState() const;
 
 public Q_SLOTS:
 
@@ -127,7 +141,7 @@ public Q_SLOTS:
                  ElisaUtils::PlayListEnqueueMode enqueueMode,
                  ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay);
 
-    void setRepeatPlay(bool value);
+    void setRepeatMode(Repeat value);
 
     void setShufflePlayList(bool value);
 
@@ -135,7 +149,7 @@ public Q_SLOTS:
 
     void skipNextTrack();
 
-    void skipPreviousTrack();
+    void skipPreviousTrack(qint64 position);
 
     void switchTo(int row);
 
@@ -174,7 +188,7 @@ Q_SIGNALS:
 
     void nextTrackDataChanged();
 
-    void repeatPlayChanged();
+    void repeatModeChanged();
 
     void shufflePlayListChanged();
 
@@ -202,6 +216,8 @@ Q_SIGNALS:
 
     void hideUndoNotification();
 
+    void seek(qint64 position);
+
 private Q_SLOTS:
 
     void sourceRowsAboutToBeInserted(const QModelIndex &parent, int start, int end);
